import assert from "assert";
import chalk from "chalk";
import pLimit from "p-limit";
export class Runner {
    constructor(d, limiter) {
        this._addedTasks = new WeakMap();
        this._errored = false;
        this._startTimes = new WeakMap();
        this._limiter = limiter ?? pLimit(d.numCPUs);
        this._d = d;
    }
    async runTasks(...tasks) {
        await Promise.all(tasks.map((task) => {
            const cached = this._addedTasks.get(task);
            if (cached) {
                return cached;
            }
            const promise = this._runTask(task);
            this._addedTasks.set(task, promise);
            return promise;
        }));
    }
    async _runTask(task) {
        const { dependencies, run } = task.options;
        if (dependencies) {
            await this.runTasks(...dependencies);
        }
        if (!run) {
            return;
        }
        return this._limiter(async () => {
            try {
                this.onTaskStart(task);
                await run();
                this.onTaskFinish(task);
            }
            catch (e) {
                this.onTaskError(task, e);
                throw e;
            }
        });
    }
    onTaskStart(task) {
        this._startTimes.set(task, Date.now());
        if (this._errored) {
            return; // Skip logging.
        }
        this._d.log(`Starting ${chalk.blue(task.options.name)}`);
    }
    onTaskFinish(task) {
        if (this._errored) {
            return; // Skip logging.
        }
        const took = Date.now() - checkDefined(this._startTimes.get(task));
        this._d.log(`Finished ${chalk.green(task.options.name)} in ${this._d.prettyMilliseconds(took)}`);
    }
    onTaskError(task, e) {
        if (this._errored) {
            return; // Skip logging.
        }
        this._errored = true;
        const took = Date.now() - checkDefined(this._startTimes.get(task));
        this._d.error(`Error in ${chalk.red(task.options.name)} in ${this._d.prettyMilliseconds(took)}\n${e}`);
    }
}
function checkDefined(value) {
    assert(value);
    return value;
}
//# sourceMappingURL=runner.js.map